@@ -45,7 +45,7 @@ xml_children <- function(x) {
 #' @rdname xml_children
 xml_child <- function(x, search = 1, ns = xml_ns(x)) {
   if (length(search) != 1) {
-    stop("`search` must be of length 1", call. = FALSE)
+    cli::cli_abort("{.arg {search}} must be of length 1.")
   }
 
   if (is.numeric(search)) {
@@ -53,7 +53,7 @@ xml_child <- function(x, search = 1, ns = xml_ns(x)) {
   } else if (is.character(search)) {
     xml_find_first(x, xpath = paste0("./", search), ns = ns)
   } else {
-    stop("`search` must be `numeric` or `character`", call. = FALSE)
+    cli::cli_abort("{.arg search} must be `numeric` or `character`.")
   }
 }
 
@@ -100,25 +100,7 @@ xml_parent.xml_nodeset <- function(x) {
 #' @export
 #' @rdname xml_children
 xml_length <- function(x, only_elements = TRUE) {
-  UseMethod("xml_length")
-}
-
-#' @export
-xml_length.xml_missing <- function(x, only_elements = TRUE) {
-  0L
-}
-
-#' @export
-xml_length.xml_node <- function(x, only_elements = TRUE) {
-  .Call(node_length, x$node, only_elements)
-}
-
-#' @export
-xml_length.xml_nodeset <- function(x, only_elements = TRUE) {
-  if (length(x) == 0)
-    return(0L)
-
-  vapply(x, xml_length, only_elements = only_elements, FUN.VALUE = integer(1))
+  .Call(node_length, x, only_elements)
 }
 
 #' @export
