@@ -1,8 +1,7 @@
-#define R_NO_REMAP
 #include <Rinternals.h>
-#undef R_NO_REMAP
-
+#include <libxml/encoding.h> // xmlCharEncoding, xmlGetCharEncodingName, xmlParseCharEncoding
 #include <libxml/parser.h>
+#include <libxml/xmlstring.h> // xmlChar, xmlStrdup
 #include <libxml/HTMLparser.h>
 #include "xml2_types.h"
 #include "xml2_utils.h"
@@ -127,7 +126,7 @@ extern "C" SEXP  xml_parse_options_() {
     "pedantic error reporting",
     "remove blank nodes",
     "use the SAX1 interface internally",
-    "Implement XInclude substitition",
+    "Implement XInclude substitution",
     "Forbid network access",
     "Do not reuse the context dictionary",
     "remove redundant namespaces declarations",
@@ -294,11 +293,14 @@ extern "C" SEXP  doc_url(SEXP doc_sxp) {
 extern "C" SEXP  doc_new(SEXP version_sxp, SEXP encoding_sxp) {
 
   const char* encoding = CHAR(STRING_ELT(encoding_sxp, 0));
+  const xmlCharEncoding enc = xmlParseCharEncoding(encoding);
+  if(enc < 0){
+    Rf_error("Requested unsupported encoding '%s'", encoding);
+  }
 
   BEGIN_CPP
   XPtrDoc x(xmlNewDoc(asXmlChar(version_sxp)));
-  xmlCharEncodingHandlerPtr p = xmlFindCharEncodingHandler(encoding);
-  x->encoding = xmlStrdup(reinterpret_cast<const xmlChar *>(p->name));
+  x->encoding = xmlStrdup(reinterpret_cast<const xmlChar *>(xmlGetCharEncodingName(enc)));
   return SEXP(x);
   END_CPP
 }
